[llvm-commits] CVS: llvm/tools/bugpoint/BugDriver.cpp BugDriver.h OptimizerDriver.cpp bugpoint.cpp

Reid Spencer reid at x10sys.com
Thu Dec 22 12:03:07 PST 2005



Changes in directory llvm/tools/bugpoint:

BugDriver.cpp updated: 1.42 -> 1.43
BugDriver.h updated: 1.41 -> 1.42
OptimizerDriver.cpp updated: 1.33 -> 1.34
bugpoint.cpp updated: 1.26 -> 1.27
---
Log message:

For PR351: http://llvm.cs.uiuc.edu/PR351 :
Generally, remove use of fork/exec from bugpoint in favor of the portable
sys::Program::ExecuteAndWait method. This change requires two new options
to bugpoint to tell it that it is running in "child" mode. In this mode,
it reads its input and runs the passes. The result code signals to the 
parent instance of bugpoint what happened (success, fail, crash).

This change should make bugpoint usable on Win32 systems.


---
Diffs of the changes:  (+97 -65)

 BugDriver.cpp       |   34 ++++++++++------
 BugDriver.h         |    9 +++-
 OptimizerDriver.cpp |  110 +++++++++++++++++++++++++++-------------------------
 bugpoint.cpp        |    9 +++-
 4 files changed, 97 insertions(+), 65 deletions(-)


Index: llvm/tools/bugpoint/BugDriver.h
diff -u llvm/tools/bugpoint/BugDriver.h:1.41 llvm/tools/bugpoint/BugDriver.h:1.42
--- llvm/tools/bugpoint/BugDriver.h:1.41	Mon Aug  1 21:16:17 2005
+++ llvm/tools/bugpoint/BugDriver.h	Thu Dec 22 14:02:55 2005
@@ -47,13 +47,14 @@
   AbstractInterpreter *Interpreter;   // How to run the program
   CBE *cbe;
   GCC *gcc;
+  bool run_as_child;
 
   // FIXME: sort out public/private distinctions...
   friend class ReducePassList;
   friend class ReduceMisCodegenFunctions;
 
 public:
-  BugDriver(const char *toolname);
+  BugDriver(const char *toolname, bool as_child);
 
   const std::string &getToolName() const { return ToolName; }
 
@@ -71,7 +72,8 @@
   }
 
   /// run - The top level method that is invoked after all of the instance
-  /// variables are set up from command line arguments.
+  /// variables are set up from command line arguments. The \p as_child argument
+  /// indicates whether the driver is to run in parent mode or child mode.
   ///
   bool run();
 
@@ -249,6 +251,9 @@
     return runPasses(PassesToRun, Filename, DeleteOutput);
   }
 
+  /// runAsChild - The actual "runPasses" guts that runs in a child process.
+  int runPassesAsChild(const std::vector<const PassInfo*> &PassesToRun);
+
   /// initializeExecutionEnvironment - This method is used to set up the
   /// environment for executing LLVM programs.
   ///


Index: llvm/tools/bugpoint/OptimizerDriver.cpp
diff -u llvm/tools/bugpoint/OptimizerDriver.cpp:1.33 llvm/tools/bugpoint/OptimizerDriver.cpp:1.34
--- llvm/tools/bugpoint/OptimizerDriver.cpp:1.33	Thu Jul  7 22:08:58 2005
+++ llvm/tools/bugpoint/OptimizerDriver.cpp	Thu Dec 22 14:02:55 2005
@@ -19,10 +19,6 @@
 // independent code co-exist via conditional compilation until it is verified
 // that the new code works correctly on Unix.
 
-#ifdef _MSC_VER
-#define PLATFORMINDEPENDENT
-#endif
-
 #include "BugDriver.h"
 #include "llvm/Module.h"
 #include "llvm/PassManager.h"
@@ -30,15 +26,18 @@
 #include "llvm/Bytecode/WriteBytecodePass.h"
 #include "llvm/Target/TargetData.h"
 #include "llvm/Support/FileUtilities.h"
+#include "llvm/Support/CommandLine.h"
 #include "llvm/System/Path.h"
+#include "llvm/System/Program.h"
 #include <fstream>
-#ifndef PLATFORMINDEPENDENT
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/wait.h>
-#endif
 using namespace llvm;
 
+namespace {
+  // ChildOutput - This option captures the name of the child output file that
+  // is set up by the parent bugpoint process
+  cl::opt<std::string> ChildOutput("child-output", cl::ReallyHidden);
+}
+
 /// writeProgramToFile - This writes the current "Program" to the named bytecode
 /// file.  If an error occurs, true is returned.
 ///
@@ -86,14 +85,14 @@
   std::cout << getPassesString(PassesToRun) << "\n";
 }
 
-static void RunChild(Module *Program,const std::vector<const PassInfo*> &Passes,
-                     const std::string &OutFilename) {
+int BugDriver::runPassesAsChild(const std::vector<const PassInfo*> &Passes) {
+
   std::ios::openmode io_mode = std::ios::out | std::ios::trunc |
                                std::ios::binary;
-  std::ofstream OutFile(OutFilename.c_str(), io_mode);
+  std::ofstream OutFile(ChildOutput.c_str(), io_mode);
   if (!OutFile.good()) {
-    std::cerr << "Error opening bytecode file: " << OutFilename << "\n";
-    exit(1);
+    std::cerr << "Error opening bytecode file: " << ChildOutput << "\n";
+    return 1;
   }
 
   PassManager PM;
@@ -115,6 +114,8 @@
 
   // Run all queued passes.
   PM.run(*Program);
+
+  return 0;
 }
 
 /// runPasses - Run the specified passes on Program, outputting a bytecode file
@@ -128,60 +129,67 @@
 bool BugDriver::runPasses(const std::vector<const PassInfo*> &Passes,
                           std::string &OutputFilename, bool DeleteOutput,
                           bool Quiet) const{
+  // setup the output file name
   std::cout << std::flush;
   sys::Path uniqueFilename("bugpoint-output.bc");
   uniqueFilename.makeUnique();
   OutputFilename = uniqueFilename.toString();
 
-#ifndef PLATFORMINDEPENDENT
-  pid_t child_pid;
-  switch (child_pid = fork()) {
-  case -1:    // Error occurred
-    std::cerr << ToolName << ": Error forking!\n";
-    exit(1);
-  case 0:     // Child process runs passes.
-    RunChild(Program, Passes, OutputFilename);
-    exit(0);  // If we finish successfully, return 0!
-  default:    // Parent continues...
-    break;
-  }
-
-  // Wait for the child process to get done.
-  int Status;
-  if (wait(&Status) != child_pid) {
-    std::cerr << "Error waiting for child process!\n";
-    exit(1);
-  }
+  // set up the input file name
+  sys::Path inputFilename("bugpoint-input.bc");
+  inputFilename.makeUnique();
+  std::ios::openmode io_mode = std::ios::out | std::ios::trunc |
+                               std::ios::binary;
+  std::ofstream InFile(inputFilename.c_str(), io_mode);
+  if (!InFile.good()) {
+    std::cerr << "Error opening bytecode file: " << inputFilename << "\n";
+    return(1);
+  }
+  WriteBytecodeToFile(Program,InFile,false);
+  InFile.close();
+
+  // setup the child process' arguments
+  const char** args = (const char**)
+    alloca(sizeof(const char*)*(Passes.size()+10));
+  int n = 0;
+  args[n++] = ToolName.c_str();
+  args[n++] = "-as-child";
+  args[n++] = "-child-output";
+  args[n++] = OutputFilename.c_str();
+  std::vector<std::string> pass_args;
+  for (std::vector<const PassInfo*>::const_iterator I = Passes.begin(),
+       E = Passes.end(); I != E; ++I )
+    pass_args.push_back( std::string("-") + (*I)->getPassArgument() );
+  for (std::vector<std::string>::const_iterator I = pass_args.begin(),
+       E = pass_args.end(); I != E; ++I )
+    args[n++] = I->c_str();
+  args[n++] = inputFilename.c_str();
+  args[n++] = 0;
 
-  bool ExitedOK = WIFEXITED(Status) && WEXITSTATUS(Status) == 0;
-#else
-  bool ExitedOK = false;
-#endif
+  sys::Path prog(sys::Program::FindProgramByName(ToolName));
+  int result = sys::Program::ExecuteAndWait(prog,args);
 
   // If we are supposed to delete the bytecode file or if the passes crashed,
   // remove it now.  This may fail if the file was never created, but that's ok.
-  if (DeleteOutput || !ExitedOK)
+  if (DeleteOutput || result != 0)
     sys::Path(OutputFilename).eraseFromDisk();
 
-#ifndef PLATFORMINDEPENDENT
+  // Remove the temporary input file as well
+  inputFilename.eraseFromDisk();
+
   if (!Quiet) {
-    if (ExitedOK)
+    if (result == 0)
       std::cout << "Success!\n";
-    else if (WIFEXITED(Status))
-      std::cout << "Exited with error code '" << WEXITSTATUS(Status) << "'\n";
-    else if (WIFSIGNALED(Status))
-      std::cout << "Crashed with signal #" << WTERMSIG(Status) << "\n";
-#ifdef WCOREDUMP
-    else if (WCOREDUMP(Status))
+    else if (result > 0)
+      std::cout << "Exited with error code '" << result << "'\n";
+    else if (result < 0)
+      std::cout << "Crashed with signal #" << abs(result) << "\n";
+    if (result & 0x01000000)
       std::cout << "Dumped core\n";
-#endif
-    else
-      std::cout << "Failed for unknown reason!\n";
   }
-#endif
 
   // Was the child successful?
-  return !ExitedOK;
+  return result != 0;
 }
 
 


Index: llvm/tools/bugpoint/bugpoint.cpp
diff -u llvm/tools/bugpoint/bugpoint.cpp:1.26 llvm/tools/bugpoint/bugpoint.cpp:1.27
--- llvm/tools/bugpoint/bugpoint.cpp:1.26	Sun Oct 23 20:05:53 2005
+++ llvm/tools/bugpoint/bugpoint.cpp	Thu Dec 22 14:02:55 2005
@@ -24,6 +24,13 @@
 #include "llvm/System/Signals.h"
 using namespace llvm;
 
+// AsChild - Specifies that this invocation of bugpoint is being generated
+// from a parent process. It is not intended to be used by users so the 
+// option is hidden.
+static cl::opt<bool> 
+  AsChild("as-child", cl::desc("Run bugpoint as child process"), 
+          cl::ReallyHidden);
+
 static cl::list<std::string>
 InputFilenames(cl::Positional, cl::OneOrMore,
                cl::desc("<input llvm ll/bc files>"));
@@ -49,7 +56,7 @@
   sys::PrintStackTraceOnErrorSignal();
   sys::SetInterruptFunction(BugpointInterruptFunction);
   
-  BugDriver D(argv[0]);
+  BugDriver D(argv[0],AsChild);
   if (D.addSources(InputFilenames)) return 1;
   D.addPasses(PassList.begin(), PassList.end());
 






More information about the llvm-commits mailing list
Index: llvm/tools/bugpoint/BugDriver.cpp
diff -u llvm/tools/bugpoint/BugDriver.cpp:1.42 llvm/tools/bugpoint/BugDriver.cpp:1.43
--- llvm/tools/bugpoint/BugDriver.cpp:1.42	Thu Apr 21 18:59:23 2005
+++ llvm/tools/bugpoint/BugDriver.cpp	Thu Dec 22 14:02:55 2005
@@ -62,9 +62,9 @@
   return Result;
 }
 
-BugDriver::BugDriver(const char *toolname)
+BugDriver::BugDriver(const char *toolname, bool as_child)
   : ToolName(toolname), ReferenceOutputFile(OutputFile),
-    Program(0), Interpreter(0), cbe(0), gcc(0) {}
+    Program(0), Interpreter(0), cbe(0), gcc(0), run_as_child(as_child) {}
 
 
 /// ParseInputFile - Given a bytecode or assembly input filename, parse and
@@ -97,13 +97,15 @@
   // Load the first input file...
   Program = ParseInputFile(Filenames[0]);
   if (Program == 0) return true;
-  std::cout << "Read input file      : '" << Filenames[0] << "'\n";
+  if (!run_as_child)
+    std::cout << "Read input file      : '" << Filenames[0] << "'\n";
 
   for (unsigned i = 1, e = Filenames.size(); i != e; ++i) {
     std::auto_ptr<Module> M(ParseInputFile(Filenames[i]));
     if (M.get() == 0) return true;
 
-    std::cout << "Linking in input file: '" << Filenames[i] << "'\n";
+    if (!run_as_child)
+      std::cout << "Linking in input file: '" << Filenames[i] << "'\n";
     std::string ErrorMessage;
     if (Linker::LinkModules(Program, M.get(), &ErrorMessage)) {
       std::cerr << ToolName << ": error linking in '" << Filenames[i] << "': "
@@ -112,7 +114,8 @@
     }
   }
 
-  std::cout << "*** All input ok\n";
+  if (!run_as_child)
+    std::cout << "*** All input ok\n";
 
   // All input files read successfully!
   return false;
@@ -124,12 +127,21 @@
 /// variables are set up from command line arguments.
 ///
 bool BugDriver::run() {
-  // The first thing that we must do is determine what the problem is. Does the
-  // optimization series crash the compiler, or does it produce illegal code?
-  // We make the top-level decision by trying to run all of the passes on the
-  // the input program, which should generate a bytecode file.  If it does
-  // generate a bytecode file, then we know the compiler didn't crash, so try
-  // to diagnose a miscompilation.
+  // The first thing to do is determine if we're running as a child. If we are,
+  // then what to do is very narrow. This form of invocation is only called
+  // from the runPasses method to actually run those passes in a child process.
+  if (run_as_child) {
+    // Execute the passes
+    return runPassesAsChild(PassesToRun);
+  }
+
+  // If we're not running as a child, the first thing that we must do is 
+  // determine what the problem is. Does the optimization series crash the 
+  // compiler, or does it produce illegal code?  We make the top-level 
+  // decision by trying to run all of the passes on the the input program, 
+  // which should generate a bytecode file.  If it does generate a bytecode 
+  // file, then we know the compiler didn't crash, so try to diagnose a 
+  // miscompilation.
   if (!PassesToRun.empty()) {
     std::cout << "Running selected passes on program to test for crash: ";
     if (runPasses(PassesToRun))

